/*---------------------------------------------------------------------------------------------------------------
/* PROGRAM	: Example1_Explore_Linked_Dataset
/* AUTHORS	: Jennifer Unangst / Feng Yu
/* DATE		: 07/31/2020
/* PURPOSE	: Demonstrate how to use a PDS-MEPS linked dataset and explore representativeness of PDS
/*            trial patients compared to U.S. cancer survivors. Analyses conducted following this 
/*            example should be considered exploratory and best suited for the purpose of becoming familiar
/*            with the linked dataset. Variance estimates produced using only the linked dataset will
/*            not account for the full MEPS sample design. Please see program examples #2-3 for full scale       
/*            analysis.
/*
/* INPUT	: Linked PDS-MEPS dataset (e.g., prostate144.sas7bdat)         
/*
/* NOTES    : 1) MEPS survey data is collected using a complex sample design that employs unequal selection 
/*            	 probabilities, clustering, and stratification (not a simple random sample). To correctly 
/*               estimate variances for MEPS data, users must run analysis using a statistical software
/*               designed for complex survey data (e.g., SUDAAN software, SAS survey procedures, STATA)
/*               and using a dataset that represents all strata and clusters in the MEPS sample (see Example2).
/*
/*            2) Each year of MEPS is designed to support inference to the U.S. population on its own.
/*               For analyses that pool MEPS respondents across MEPS years, the person-level weights from
/*               each MEPS year should be divided by the number of years used in the analysis.
/*
/*            3) For subpopulation analyses (e.g. prostate cancer survivors under 60 years of age), data
/*               users should employ the DOMAIN statement for SAS survey procedures or the SUBPOPN statement 
/*               in SUDAAN, rather than subset data to the subpopulation of interest.
/*
/* PDS = Project Data Sphere
/* MEPS = Medical Expenditure Panel Survey 
/*---------------------------------------------------------------------------------------------------------------*/
options nofmterr validvarname=v7 compress=yes;

*specify location and name of PDS-MEPS linked dataset;
%let dataDir   = /rtpnfil03/rtpnfil03_vol7/DataSphere2/Deliver/Prostat_AstraZe_2009_144; 
%let dataName  = linked_prostate144; 

libname dataDir "&dataDir";

/* see codebook that accompanies linked dataset for variable definitions */
proc format ;
	value _POVCATF 
       1 = " 1 POOR/NEGATIVE"
       2 = " 2 NEAR POOR"
       3 = " 3 LOW INCOME" 
       4 = " 4 MIDDLE INCOME"
       5 = " 5 HIGH INCOME" 
	;
	value _HIDEGYRNF
	  -9 = "-9 NOT ASCERTAINED"
      -8 = "-8 DK" 
	  -1 = "-1 NOT APPLICABLE"
       1 = " 1 NO DEGREE"
       2 = " 2 GED OR HIGH SCHOOL DIPLOMA"
       3 = " 3 BACHELOR'S DEGREE"
       4 = " 4 MASTER'S AND DOCTORATE DEGREE" 
       5 = " 5 OTHER DEGREE" 
	;
	value _INSCOVF
       1 = " 1 ANY PRIVATE"
       2 = " 2 PUBLIC ONLY"
       3 = " 3 UNINSURED" 
	;
	value _MARRYF
	  -8 = "-8 DK"
       1 = " 1 MARRIED"
       2 = " 2 WIDOWED"
       3 = " 3 DIVORCED" 
       4 = " 4 SEPARATED"
       5 = " 5 NEVER MARRIED" 
	;
	value _ADINSA42F
	  -9 = "-9 NOT ASCERTAINED"
      -1 = "-1 INAPPLICABLE"
       1 = " 1 DISAGREE STRONGLY"
       2 = " 2 DISAGREE SOMEWHAT"
       3 = " 3 UNCERTAIN"
       4 = " 4 AGREE SOMEWHAT"
       5 = " 5 AGREE STRONGLY"
	;		
	value _ADINSB42F
	  -9 = "-9 NOT ASCERTAINED"
      -1 = "-1 INAPPLICABLE"
       1 = " 1 DISAGREE STRONGLY"
       2 = " 2 DISAGREE SOMEWHAT"
       3 = " 3 UNCERTAIN"
       4 = " 4 AGREE SOMEWHAT"
       5 = " 5 AGREE STRONGLY"
	;
	value _AGECATF
       1 = " 1 AGE 45-59"
       2 = " 2 AGE 60-64" 
       3 = " 3 AGE 65-69"
       4 = " 4 AGE 70-74"
       5 = " 5 AGE 75-79"
       6 = " 6 AGE 80+" 
	;
	value _RACEF
       1 = " 1 WHITE"
       3 = " 3 HISPANIC"
       4 = " 4 ASIAN/PACIF"
       6 = " 6 OTHER/MULTI (INCLUDES BLACK/AFR AMER)"
	;
	value _LINKSTATUSF
	 1 = "1 PDS-MEPS LINK"
	 2 = "2 UNLINKED MEPS CASE"
	 ;
	value _LINKEDF
	 1 = "1 MEPS CANCER SURVIVOR LINKED WITH >=1 PDS PATIENT"
	 2 = "2 MEPS CANCER SURVIVOR DID NOT LINK WITH ANY PDS PATIENTS"
	 ;
quit;

************************************************************
* Get familiar with linked dataset
************************************************************;

/* Read in PDS-MEPS linked dataset */
proc contents data = dataDir.&dataName varnum; run;

data linkedData;
	set dataDir.&dataName;
run;

/* Demonstrate that dataset is unique by MEPSID and PDSID */
proc sort data = linkedData dupout = chkDups nodupkey; by MEPSID PDSID; run;

data _null_;
	set chkDups;
	if _n_=1 then put "WARNING[user-defined]: Duplicate by MEPSID and PDSID " ;
run;

/* Demonstrate how to distinguish observations that represent a PDS-MEPS linkage versus an unlinked MEPS case */
title "Observations with LinkStatus = 1 represent MEPS-PDS linkages; LinkStatus = 2 are unlinked MEPS cancer survivors";
proc freq data = linkedData ;
format linkstatus _linkstatusf. ;
tables linkStatus / list missing;
run;
title "";

/* Many-to-many linkages exist in the dataset: review the number of PDS patients linked to each MEPS cancer survivor */
proc freq data = linkedData noprint;
where linkStatus = 1;
tables MEPSID / out = countPDSlink ;
run;

title "Number of PDS Patients Linked to MEPS Cancer Survivors ";
proc univariate data = countPDSlink; var count; run;
title "";

***********************************************************************
* Create dataset for exploratory analysis 
***********************************************************************;

/* We will compare linked and unlinked MEPS cancer survivors to assess representativeness of 
   the clinical trial patients, so we must account for duplicate instances of each MEPSID 
   in the dataset (duplicates occur when a MEPS case links with more than one PDS patient).
   Note: When we de-duplicate by MEPSID, the dataset no longer contains linkages between
   MEPS cancer survivors and PDS patients, so we drop PDSID variable. */

proc sort data = linkedData; by MEPSID; run;

data NoDup;
	set linkedData;
	by MEPSID;
	if last.MEPSID;
	drop PDSID; 
run;

title "Age Range of MEPS Cancer Survivors Represented in the Analysis";
proc means data = Nodup n nmiss min max;
var age ;
run;

title "MEPS Years with at Least One Cancer Survivor of Interest";
title2 "Number of years is used for pooling adjustment below";
proc freq data = NoDup;
tables year / list missing;
run;
title2 "";

data analysis;
 set NoDup;

 **create the analysis weights for the pooled data;
 PERWTFP17 = PERWTF/17;
 SAQWTFP17 = SAQWTF/17;

 **recode variables for analysis as needed 
  (here, set REF/DK responses to SAS missings);
 array varname{*} hidegyrn       marry      adinsa42      adinsb42 ;
 array origvar{*} hidegyrn_orig  marry_orig adinsa42_orig adinsb42_orig;

 do i = 1 to dim(varname);
 	origvar[i] = varname[i];
	if origvar[i] < 0 then varname[i]=.;
 end;

 drop i;

 if      linkstatus = 1 then linked = 1; 
 else if linkstatus = 2 then linked = 2;

 **create numeric ID variable for analysis in SUDAAN;
 sudCaseID = _n_;
run;

title "Check derived variable recodes ";
proc freq data=analysis;
 format linked _linkedf. agecat _agecatf. hidegyrn _hidegyrnf. inscov _inscovf. marry _marryf.
        povcat _povcatf. race _racef. adinsa42 _adinsa42f. adinsb42 _adinsb42f.;
 tables linked 
		agecat
	    hidegyrn_orig * hidegyrn 
        inscov
        marry_orig * marry 
        povcat
	    race  
        adinsa42_orig * adinsa42 
        adinsb42_orig * adinsb42 / list missing;
run;
       
******************************************************
***Compare linked vs unlinked MEPS cancer survivors***
******************************************************;
/* Note: Standard errors and confidence intervals produced using these examples may not be accurate, 
   because the set of MEPS cancer survivors in the linked dataset does not represent all
   strata and clusters from the full MEPS sample design. Users should refer to programs #2-3 for 
   how to produce standard errors and hypothesis tests that account for the full MEPS sample design.
   Point estimates (e.g., percentages or means) produced using these examples are correct, since
   they employ MEPS sampling weights.   */

*-------------------------------------------------------------
* Analysis using SAS 
*-------------------------------------------------------------;
title "SAS SURVEYFREQ - COMPARE LINKED VS UNLINKED MEPS CANCER SURVIVORS (PERWT VARIABLES)";
proc surveyfreq data=analysis;
   tables  linked * agecat 
           linked * hidegyrn 
           linked * inscov 
           linked * marry 
           linked * race  / row ;
   strata  varstra9617; 
   cluster varpsu9617; 
   weight  PERWTFP17;  /* a pooled weight to produce annual averages based on combined years of data */
run;

title "SAS SURVEYFREQ - COMPARE LINKED VS UNLINKED MEPS CANCER SURVIVORS (SAQWT VARIABLES)";
proc surveyfreq data=analysis;
   tables  linked * adinsa42 
           linked * adinsb42  / row ;
   strata  varstra9617; 
   cluster varpsu9617; 
   weight  SAQWTFP17;  /* a pooled weight to produce annual averages based on combined years of data */
run;

*-------------------------------------------------------------
* Analysis using SUDAAN
*-------------------------------------------------------------;
/* Data user must have SAS-callable SUDAAN installed to run this example */

proc sort data=analysis; by varstra9617 varpsu9617; run;

title "SUDAAN - COMPARE LINKED VS UNLINKED MEPS CANCER SURVIVORS (PERWT VARIABLES)";
proc crosstab data=analysis filetype=sas design=wr ;
 nest     varstra9617 varpsu9617 / missunit; /* variance stratum and cluster from MEPS sample design */
 weight   PERWTFP17;  			  			 /* a pooled weight to produce annual averages based on combined years of data */
 idvar    sudcaseID;
 subgroup agecat hidegyrn inscov marry race linked / include=nonmissing;
 levels   6      5        3      5     5    2;
 print    nsum colper secol  / ;
 tables   agecat * linked
          hidegyrn * linked
          inscov * linked
          marry * linked
          race * linked ;
 run;

title "SUDAAN - COMPARE LINKED VS UNLINKED MEPS CANCER SURVIVORS (SAQWT VARIABLES)";
proc crosstab data=analysis filetype=sas design=wr ;
 nest     varstra9617 varpsu9617 / missunit; /* variance stratum and cluster from MEPS sample design */
 weight   SAQWTFP17;  			  			 /* variables from the SAQ questionnaire need the SAQ weight adjusted for pooling */
 idvar    sudcaseID;
 subgroup adinsa42 adinsb42 linked / include=nonmissing;
 levels   5        5        2;
 print    nsum colper secol ;
 tables   adinsa42 * linked
          adinsb42 * linked ;
 run;
 