/*---------------------------------------------------------------------------------------------------------------
/* PROGRAM	: Example2_Process_MEPS_Source_Files
/* AUTHOR	: Jennifer Unangst / Feng Yu
/* DATE		: 08/05/2020
/*
/* PURPOSE	: Read the 2000 - 2016 MEPS Full Year Consolidated Household Component public use files 
/*            and merge with the MEPS pooled variance estimation file. Demonstrate how to append MEPS 
/*            survey variables and pooled variance structure to a PDS-MEPS linked dataset. 
/*            Create an ID variable that is unique across MEPS years.
/*            
/*            This code processes several MEPS insurance variables as an example: MCAREX, MCAIDX, PRVHMO, TRIATX.
/*            It does not process all MEPS variables available from the MEPS public use datasets.
/*
/* INPUT	: 2000-2016 MEPS Full Year Conslidated Household Component Public Use Datasets
/*            1997-2017 Pooled Linkage Variance Estimation File 
/*
/* OUTPUT	: This program will output a pooled dataset of MEPS respondents from 2000-2016. The pooled
/*            dataset can be used to append MEPS survey variables to a PDS-MEPS linked dataset and
/*            to produce standard errors that reflect the full MEPS complex sample design.
/*            
/*            pooled_meps00to16.sas7bdat
/*
/* NOTES    : MEPS public use data can be downloaded from the following website:
/*            https://meps.ahrq.gov/data_stats/download_data_files.jsp
/*
/*            Prior to running the program, each Full Year Consolidated dataset was downloaded and 
/*            saved in a folder with same name as the file name (e.g., dataset H163 was saved in
/*            a directory called H163. SAS transport files were downloaded from the MEPS website and
/*            used to convert the MEPS data to SAS datasets.
/*           
/*            Be advised that MEPS variable names and definitions may differ across years of the survey
/*            implementation. Please refer to the documentation that accompanies each Household Component  
/*            data file to confirm variable definitions.
/*---------------------------------------------------------------------------------------------------------------*/
options nofmterr validvarname=v7 compress=yes;

*** specify working directory;
%let basepath = /rtpnfil03/rtpnfil03_vol7/DataSphere2;

*** specify destination for SAS versions of MEPS public use datasets;
libname mepsdat "&basepath/SAS_data/MEPS";

*** specify destination for pooled MEPS dataset; 
libname outdat  "&basepath/SAS_pgms/05_Data_Deliverable/Prostat_AstraZe_2009_144/SampleCode/data/";


proc format ;
***************************************************************
* Raw Variable Formats from MEPS Codebooks
***************************************************************;
value HISPANXF
  1 = "Hispanic"
  2 = "Non-Hispanic"
 ;
/*RACEX defined based on 2000 - 2001 MEPS*/
value RACEX0001F
  1 = "American Indian"
  2 = "Aleut, Eskimo"
  3 = "Asian or Pacif Islander"
  4 = "Black"
  5 = "White"
;
/*RACEX defined based on 2002 - 2011 MEPS*/
value RACEX0211F
  3 = "American Indian / Alaska Native"
  6 = "Multi race"
  5 = "Native HI / Pacif Island"
  4 = "Asian"
  2 = "Black"
  1 = "White"
;
/*RACEX defined based on 2012 - 2016 MEPS*/
value RACEV1XF
  1 = "White"
  2 = "Black"
  3 = "American Indian / Alaskan Native"
  4 = "Asian"
  5 = "Native HI / Pacif Island"
  6 = "Multi race"
;
value _SEXF
1= "1 Male"
2= "2 Female"
;
***************************************************************
* Derived Variable Formats (Specified by User)
***************************************************************;
value _mcarexf
-1= "-1 INAPPLICABLE "
1= "1 Yes "
2= "2 No"
;
value _mcaidxf
-1= "-1 INAPPLICABLE "
1= "1 Yes "
2= "2 No"
;
value _prvhmof
-9 = "-9 COV BY PRIVATE - HMO NOT ASCERTAINED" 
-1 = "-1 INAPPLICABLE" 
1  = "1 COV BY PRIVATE HMO" 
2  = "2 COV BY PRIVATE INS - NOT HMO" 
3  = "3 NOT COV BY PRIVATE INS"
;
value _triatxf
-1= "-1 INAPPLICABLE "
1= "1 Yes "
2= "2 No"
;
value AGECATF 
  1 = '< 5'
  2 = '5-17'
  3 = '18-24'
  4 = '25-34'
  5 = '35-44'
  6 = '45-54'
  7 = '55-64'
  8 = '65-74'
  9 = '75-84'
 10 = '85+'
 ;
value RACECATF
  1 = "White"
  2 = "Black"
  3 = "Hispanic"
  4 = "Asian/Pacif"
  5 = "Am Indian/Alaskan Native"
  6 = "Other/Multi-race"
 ;
run;

*----------------------------------------------------------------
* READ MEPS FULL YEAR HOUSEHOLD COMPONENT (HC) FILES 
*----------------------------------------------------------------;

%MACRO READ_MEPS();

  %let yearxx = %SUBSTR(&year,3,2);

	/*convert SAS transport file to temporary SAS dataset*/
	filename in1 "&basepath/MEPS_Downloads/&folder/&filename";     
	proc xcopy in=in1 out=work import;
	run;

	/*check no duplicates by person within full year household file*/
	proc sort data=&dsname out=&dsname._nodup dupout=shouldbezero nodupkey;
	  by dupersid;
	run;

	data _null_;
	  set shouldbezero;
	  if _n_=1 then put "ERROR[user-defined]: Duplicates by DUPERSID in &dsname ";
	run;

   data &dsname._full;  
     length mepsid $20 Year $4;
     set &dsname;
     
   	 year = "&year";
   
	 *------------------------------------------------------
	 * Create MEPS ID (unique merge variable across years)
	 *------------------------------------------------------;
	 if length(dupersid)>10 then put "ERROR[user-defined]: DUPERSID has length > 10";
	 MEPSID = strip(DUPERSID)||strip(year);

	 *--------------------------------------------------------------------------------
	 * Impute age based on DOB if edited/imputed age variable is missing.
	 * Day of DOB is unknown so the 15th will be assumed for all with unknown age.
	 * In 2000, age was top-coded at 90. In all later years, age was top-coded at 85.
	 *--------------------------------------------------------------------------------;

  	  length refdate $10;
      refdate  = "31DEC"||strip(&year);
      agerefdt = input(refdate,date9.);

	   if 0 <= dobmm < 10 then _dobmm = "0"||left(dobmm); else _dobmm = dobmm;
	   _dobmmyy  = catx("/",_dobmm,"15",dobyy);  
       _dobmmyyn = input(_dobmmyy,mmddyy10.);
	   
	   if &year = "2000" then maxage = 90;
	   else				      maxage = 85;

	   if      age&yearxx.x >= 0 then _agevar = age&yearxx.x;
       else if age&yearxx.x = -1 then _agevar = min(int(sum(-1*_dobmmyyn,agerefdt)/365.25),maxage);

	 *--------------------------------------------------------------------------------
	 * Create age categories
	 *--------------------------------------------------------------------------------;

	   *standardize name of age variable across MEPS years;
	   AGE = _agevar;

 	   if      0  <= AGE  < 5  then AGECAT = 1; 
 	   else if 5  <= AGE <= 17 then AGECAT = 2; 
       else if 18 <= AGE <= 24 then AGECAT = 3; 
 	   else if 25 <= AGE <= 34 then AGECAT = 4; 
 	   else if 35 <= AGE <= 44 then AGECAT = 5; 
       else if 45 <= AGE <= 54 then AGECAT = 6; 
 	   else if 55 <= AGE <= 64 then AGECAT = 7; 
 	   else if 65 <= AGE <= 74 then AGECAT = 8; 
       else if 75 <= AGE <= 84 then AGECAT = 9; 
 	   else if       AGE >= 85 then AGECAT = 10;
 	   else put "WARNING[user-def]: Imputed AGE is Missing ";

	 *--------------------------------------------------------------------------------
	 * Standardize race
	 *--------------------------------------------------------------------------------;

       if year in ("2000" "2001") then do;
 		if 	    hispanx  = 1 then RACECAT = 3;  /*"HISPANIC"*/
 		else if hispanx ne 1 and &racevar. in (1 2) then RACECAT = 5; /*"AMERIND/ALASKAN"*/
 		else if hispanx ne 1 and &racevar. = 3 then RACECAT = 4; /*"ASIAN/PACIF"*/
 		else if hispanx ne 1 and &racevar. = 4 then RACECAT = 2; /*"BLACK"*/
 		else if hispanx ne 1 and &racevar. = 5 then RACECAT = 1; /*"WHITE"*/
 		else RACECAT = .;
	   end;

	   else do;
 		if 	    hispanx  = 1 then RACECAT = 3; /*"HISPANIC"*/
 		else if hispanx ne 1 and &racevar. = 3 then RACECAT = 5; /*"AMERIND/ALASKAN"*/
 		else if hispanx ne 1 and &racevar. in (4 5) then RACECAT = 4; /*"ASIAN/PACIF"*/
 		else if hispanx ne 1 and &racevar. = 2 then RACECAT = 2; /*"BLACK"*/
 		else if hispanx ne 1 and &racevar. = 1 then RACECAT = 1; /*"WHITE"*/
 		else if hispanx ne 1 and &racevar. = 6 then RACECAT = 6; /*"OTHER/MULTI"*/
 		else RACECAT = .;
	   end;
	 
	 *---------------------------------------------------------------------------------
	 * User can add other cleaning logic here 
	 *---------------------------------------------------------------------------------;

	 *** Example: Health insurance coverage variable values are defined consistently 
	     across MEPS years, but the variable names need to be standardized to strip 
	     the MEPS year from the name.  ;
	       
		mcarex = mcare&yearxx.x;
		mcaidx = mcaid&yearxx.x;
		prvhmo = prvhmo&yearxx ;
		triatx = triat&yearxx.x;
       
   	 rename &panelvar      = PANEL 
            &saqwtvar.     = SAQWTF
            &wtvar.        = PERWTF
            varpsu&fyear.  = VARPSU
            varstr&fyear.  = VARSTR
			age&yearxx.x   = AGEYYX;
   run;
   
   data &dsname._keepvar;  
     set &dsname._full;  

	  keep dupersid mepsid year sex PERWTF SAQWTF PANEL VARPSU VARSTR 
           age agecat racecat

          /*User can update list of variables to keep here*/
           mcarex mcaidx prvhmo triatx ; 
	       
   	  label 

		   mepsid = "MEPS DERIVED MEPS ID (DUPERSID||YEAR)"
		   panel  = "MEPS PANEL NUMBER"
           perwtf = "MEPS Person-level Weight"
           SAQWTF = "MEPS SAQ Person-level WEIGHT"
           SEX    = "MEPS SEX"
           VARPSU = "VARIANCE ESTIMATION PSU (SINGLE YEAR MEPS ANALYSIS)"
           VARSTR = "VARIANCE ESTIMATION STRATUM (SINGLE YEAR MEPS ANALYSIS)"
           YEAR   = "MEPS SURVEY YEAR"
		   AGE    = "MEPS IMPUTED AGE (YEARS)"
		   AGECAT = "MEPS IMPUTED AGE (CATEGORIZED)"
		   RACECAT= "MEPS RACE"

          /*User can update labels here as needed*/
		   mcarex = "COVERED BY MEDICARE - 12/31 (ED)"
      	   mcaidx = "COVERED BY MEDICAID OR SCHIP - 12/31 (ED)"
      	   prvhmo = "COVERED BY PRIVATE HMO - 12/31 "
      	   triatx = "ANY TIME COV TRICARE/CHAMPVA - 12/31"
           ;  

   run;
%MEND;

*2000 Full HH;
%let fyear     = 00;
%let year      = 2000;
%let folder    = h50;
%let filename  = H50.SSP;
%let dsname    = H50; 
%let panelvar  = panel00;
%let saqwtvar  = sqpqw00f;
%let wtvar     = perwt00f;
%let racevar = racex;
%READ_MEPS();

*2001 Full HH;
%let fyear     = 01;
%let year      = 2001;
%let folder    = h60;
%let filename  = H60.SSP;
%let dsname    = H60;
%let panelvar  = panel01;
%let saqwtvar  = saqwt01f; 
%let wtvar     = perwt01f;
%let racevar = racex;
%READ_MEPS();

*2002 Full HH;
%let fyear     = ;
%let year      = 2002;
%let folder    = h70;
%let filename  = h70.ssp;
%let dsname    = h70; 
%let panelvar  = panel02;
%let saqwtvar  = saqwt02f;
%let wtvar     = perwt02f;
%let racevar = racex;
%READ_MEPS();

*2003 Full HH;
%let fyear     = ;
%let year      = 2003;
%let folder    = h79;
%let filename  = H79.SSP;
%let dsname    = H79; 
%let panelvar  = panel03;
%let saqwtvar  = saqwt03f;
%let wtvar     = perwt03f;
%let racevar = racex;
%READ_MEPS();

*2004 Full HH;
%let fyear     = ;
%let year      = 2004;
%let folder    = h89;
%let filename  = H89.SSP;
%let dsname    = h89; 
%let panelvar  = panel04;
%let saqwtvar  = saqwt04f;
%let wtvar     = perwt04f;
%let racevar = racex;
%READ_MEPS();

*2005 Full HH;
%let fyear     = ;
%let year      = 2005;
%let folder    = h97;
%let filename  = h97.ssp;
%let dsname    = h97; 
%let panelvar  = panel;
%let saqwtvar  = saqwt05f;
%let wtvar     = perwt05f;
%let racevar = racex;
%READ_MEPS();

*2006 Full HH;
%let fyear     = ;
%let year      = 2006;
%let folder    = h105;
%let filename  = H105.SSP;
%let dsname    = h105;
%let panelvar  = panel;
%let saqwtvar  = saqwt06f; 
%let wtvar     = perwt06f;
%let racevar = racex;
%READ_MEPS();

*2007 Full HH;
%let fyear     = ;
%let year      = 2007;
%let folder    = h113;
%let filename  = h113.ssp;
%let dsname    = h113;
%let panelvar  = panel;
%let saqwtvar  = saqwt07f; 
%let wtvar     = perwt07f;
%let racevar = racex;
%READ_MEPS();

*2008 Full HH;
%let fyear     = ;
%let year      = 2008;
%let folder    = h121;
%let filename  = h121.ssp;
%let dsname    = h121; 
%let panelvar  = panel;
%let saqwtvar  = saqwt08f;
%let wtvar     = perwt08f;
%let racevar = racex;
%READ_MEPS();

*2009 Full HH;
%let fyear     = ;
%let year      = 2009;
%let folder    = h129;
%let filename  = h129.ssp;
%let dsname    = h129;
%let panelvar  = panel;
%let saqwtvar  = saqwt09f; 
%let wtvar     = perwt09f;
%let racevar = racex;
%READ_MEPS();

*2010 Full HH;
%let fyear     = ;
%let year      = 2010;
%let folder    = h138;
%let filename  = h138.ssp;
%let dsname    = h138; 
%let panelvar  = panel;
%let saqwtvar  = saqwt10f;
%let wtvar     = perwt10f;
%let racevar = racex;
%READ_MEPS();

*2011 Full HH;
%let fyear     = ;
%let year      = 2011;
%let folder    = h147;
%let filename  = h147.ssp;
%let dsname    = h147; 
%let panelvar  = panel;
%let saqwtvar  = saqwt11f;
%let wtvar     = perwt11f;
%let racevar = racex;
%READ_MEPS();

*2012 Full HH;
%let fyear     = ;
%let year      = 2012;
%let folder    = h155;
%let filename  = h155.ssp;
%let dsname    = h155; 
%let panelvar  = panel;
%let saqwtvar  = saqwt12f;
%let wtvar     = perwt12f;
%let racevar = racev1x;
%READ_MEPS();

*2013 Full HH;
%let fyear     = ;
%let year      = 2013;
%let folder    = h163;
%let filename  = h163.ssp;
%let dsname    = h163; 
%let panelvar  = panel;
%let saqwtvar  = saqwt13f;
%let wtvar     = perwt13f;
%let racevar = racev1x;
%READ_MEPS();

*2014 Full HH;
%let fyear     = ;
%let year      = 2014;
%let folder    = h171;
%let filename  = h171.ssp;
%let dsname    = h171; 
%let panelvar  = panel;
%let saqwtvar  = saqwt14f;
%let wtvar     = perwt14f;
%let racevar = racev1x;
%READ_MEPS();

*2015 Full HH;
%let fyear     = ;
%let year      = 2015;
%let folder    = h181;
%let filename  = h181.ssp;
%let dsname    = h181; 
%let saqwtvar  = saqwt15f;
%let wtvar     = perwt15f;
%let racevar = racev1x;
%READ_MEPS();

*2016 Full HH;
%let fyear     = ;
%let year      = 2016;
%let folder    = h192;
%let filename  = h192.ssp;
%let dsname    = h192;
%let panelvar  = panel;
%let saqwtvar  = saqwt16f; 
%let wtvar     = perwt16f;
%let racevar = racev1x;
%READ_MEPS();

*** check variable processing;
data qc_derived;
   set h50_full  
	   h60_full  
	   h70_full  
	   h79_full  
	   h89_full  
	   h97_full  
	   h105_full 
	   h113_full 
	   h121_full 
	   h129_full 
	   h138_full 
	   h147_full 
	   h155_full 
	   h163_full
	   h171_full 
	   h181_full
	   h192_full;
run;

title "*** Check Derived Variable ***";
proc freq data=qc_derived;  
format AGECAT AGECATF.;
tables AGEYYX * AGE * AGECAT  / list missing;
run;
title "";

proc freq data=qc_derived;  
title2 "2000 - 2001 Race";
format RACECAT RACECATF. HISPANX HISPANXF.  RACEX RACEX0001F.;
where year in ('2000' '2001');
tables HISPANX * RACEX * RACECAT / list missing;
run;

proc freq data=qc_derived;  
title2 "2002 - 2011 Race";
format RACECAT RACECATF.  HISPANX HISPANXF. RACEX RACEX0211F.;
where year not in ('2000' '2001' '2012' '2013' '2014' '2015' '2016');
tables HISPANX * RACEX * RACECAT / list missing;
run;
title ""; title2 "";

proc freq data=qc_derived;  
title2 "2012 - 2016 Race";
format RACECAT RACECATF.  HISPANX HISPANXF. RACEV1X RACEV1XF.;
where year in ('2012' '2013' '2014' '2015' '2016');
tables HISPANX * RACEV1X * RACECAT / list missing;
run;
title ""; title2 "";

************************* Create pooled analysis file *************************
*  Notes from MEPS documentation:
*
*  To facilitate analysis of subpopulations and/or low prevalence events, 
*  it may be desirable to pool together (i.e., combine) more than one year 
*  of MEPS-HC data to yield sample sizes large enough to generate reliable 
*  estimates. MEPS-HC samples in most years are not completely independent 
*  because households are drawn from the same sample geographic areas and 
*  many persons are in the sample for two consecutive years (see MEPS-HC 
*  Methodology Reports for more details at http://www.meps.ahrq.gov). 
*
*  Despite this lack of independence, it is valid to pool multiple years 
*  of MEPS-HC data and keep all observations in the analysis because each 
*  year of the MEPS-HC is designed to be nationally representative. However, 
*  to obtain appropriate standard errors when pooling years of MEPS-HC data, 
*  it is necessary to specify a common variance structure that properly 
*  reflects the complex sample design of the MEPS (see codebook for H36u17)
*******************************************************************************;

data meps00to16;
   set h50_keepvar  
	   h60_keepvar  
	   h70_keepvar  
	   h79_keepvar  
	   h89_keepvar  
	   h97_keepvar  
	   h105_keepvar 
	   h113_keepvar 
	   h121_keepvar 
	   h129_keepvar 
	   h138_keepvar 
	   h147_keepvar 
	   h155_keepvar 
	   h163_keepvar
	   h171_keepvar 
	   h181_keepvar
	   h192_keepvar;
run;

*--------------------------------------------------------------
* ATTACH POOLED VARIANCE STRUCTURE TO MEPS ANALYSIS DATASET
*--------------------------------------------------------------;

*** Read PSU and stratum indicators for pooled variance structure;
data h36u17;
  set mepsdat.h36u17;
run;

proc sort data=h36u17 dupout=shouldbezero nodupkey; by dupersid panel; run;
data _null_;
 set shouldbezero;
 if _n_=1 then put "WARNING[user-defined]: Duplicates by DUPERSID and PANEL in H36 ";
run;

*** Merge onto MEPS cases ;
proc sort data=meps00to16 ; by dupersid panel; run;
data pooled_meps;
 merge meps00to16 (in=a)
       h36u17     (in=b keep=dupersid panel stra9617 psu9617);
 by dupersid panel;
 if a and not b then put "WARNING[user-defined]: MEPS Case not in H36 " dupersid= panel=;
 if a;
 label stra9617 = "VARIANCE ESTIMATION STRATUM (POOLED MEPS ANALYSIS)"
       psu9617  = "VARIANCE ESTIMATION PSU (POOLED MEPS ANALYSIS)";
run;

*** Check no missing values for pooled sample design variables ;
proc means data=pooled_meps nmiss min max;
title "MEPS Pooled Variance Variables (Check None Missing)";
var stra9617 psu9617 perwtf saqwtf;
run;

*** Check each unique caseID from MEPS has same design parameters across years;
proc freq data=pooled_meps noprint;
 tables dupersid*panel*stra9617*psu9617 / out=qc_pooledmerge (drop=count percent);
run;

proc sort data=qc_pooledmerge dupout=shouldbezero nodupkey; by dupersid panel; run;

data _null_;
 set shouldbezero;
 if _n_=1 then put "ERROR[user-defined]: Differant Stratum and PSU for DUPERSID PANEL Value ";
run;

**output dataset;
proc sort data=pooled_meps out=outdat.pooled_meps00to16;
 by year dupersid;
run;

title "Pooled 2000-2016 MEPS Dataset (example contains a subset of available MEPS variables)";
proc contents data=outdat.pooled_meps00to16; run;
title "";