/*---------------------------------------------------------------------------------------------------------------*
/* PROGRAM	: Example3_Analyze_MEPS_Cancer_Survivors
/* AUTHOR	: Jennifer Unangst / Feng Yu
/* DATE		: 08/01/2020
/* PURPOSE	: Demonstrate how to:
/*            Produce nationally representative estimates of U.S. cancer survivors of interest
/*            Compare characteristics of linked and unlinked MEPS cancer survivors  
/*
/* INPUTS	: 1) pooled_meps00to16.sas7bdat (Pooled MEPS dataset output from Example2)
/*				 The pooled MEPS dataset is output from Example2 program. It includes all MEPS respondents
/*               from 2000-2016 and the MEPS variables you want to use in your analysis. Because all strata
/*               and clusters from the MEPS sample design are represented, this dataset supports variance
/*               estimates that account for the MEPS complex sample design.
/*
/*            2) PDS-MEPS linked dataset (e.g., prostate144.sas7bdat, downloaded from PDS website) 
/*
/* NOTES	: To produce estimates for subgroups and subpopulations of interest, the data user should not
/*            subset the POOLED_MEPS dataset. Subsetting will alter the set of clusters and strata 
/*            used by SAS and SUDAAN to estimate standard errors and confidence intervals.
/*---------------------------------------------------------------------------------------------------------------*/
options nofmterr validvarname=v7 compress=yes;

*** specify working directory;
%let basepath = /rtpnfil03/rtpnfil03_vol7/DataSphere2;

*** specify location of pooled MEPS dataset;
libname mepsdat "&basepath/SAS_pgms/05_Data_Deliverable/Prostat_AstraZe_2009_144/SampleCode/data/";

*** specify location of PDS-MEPS linked dataset; 
libname linkdat  "&basepath/Deliver/Prostat_AstraZe_2009_144";

*** specify name of PDS-MEPS linked dataset;
%let pdsfname  = prostate144;

************************************************************************************************
* Create analysis dataset
************************************************************************************************;

proc format ;
value _SEXF
1= "1 Male"
2= "2 Female"
;
value _mcarexf
-1= "-1 INAPPLICABLE "
1= "1 Yes "
2= "2 No"
;
value _mcaidxf
-1= "-1 INAPPLICABLE "
1= "1 Yes "
2= "2 No"
;
value _prvhmof
-9 = "-9 COV BY PRIVATE - HMO NOT ASCERTAINED" 
-1 = "-1 INAPPLICABLE" 
1  = "1 COV BY PRIVATE HMO" 
2  = "2 COV BY PRIVATE INS - NOT HMO" 
3  = "3 NOT COV BY PRIVATE INS"
;
value _triatxf
-1= "-1 INAPPLICABLE "
1= "1 Yes "
2= "2 No"
;
value LINKSTATUSF
 1 = "1 PDS-MEPS LINK"
 2 = "2 UNLINKED MEPS CASE"
 ;
 
value LINKSTATUS_BMIF
 1 = "1 PDS-MEPS LINK MEETS ADDL BMI CRITERION "
 2 = "2 PDS-MEPS LINK DOES NOT MEET ADDL BMI CRIT"
 3 = "3 UNLINKED MEPS CASE"
 ;
value AGECATF 
  1 = '< 5'
  2 = '5-17'
  3 = '18-24'
  4 = '25-34'
  5 = '35-44'
  6 = '45-54'
  7 = '55-64'
  8 = '65-74'
  9 = '75-84'
 10 = '85+'
 ;
value RACECATF
  1 = "White"
  2 = "Black"
  3 = "Hispanic"
  4 = "Asian/Pacif"
  5 = "Am Indian/Alaskan Native"
  6 = "Other/Multi-race"
 ;
 value PROSTATE45F
  1 = 'Yes'
  0 = 'No'
  ;
 value MATCHEDF
  1  = 'Yes'
  2  = 'No'
  -9 = 'Not Eligible for Linkage'
 ;
 value MATCHED_PROSTATE45F
  1 = 'Linked Prostate Cancer Surv (45+)'
  2 = 'Non-linked Prostate Cancer Surv (45+)'
  0 = 'Not Eligible for Linkage'
  ;
  value WHITEF
  1 = 'Yes'
  2 = 'No'
  ;
run;

*** read pooled MEPS data file
    (this includes all MEPS respondents -- necessary for variance estimates);
proc sort data=mepsdat.pooled_meps00to16 out=pooled_meps00to16; 
 by MEPSID;
run;

*** read PDS-MEPS linked dataset
   (this includes all MEPS respondents who have the same cancer type and age range as the linked PDS trial);
data linked;
	set linkdat.linked_&pdsfname ;
	keep MEPSID linkstatus linkstatus_bmi;
run;

title "PDS-MEPS Linked Dataset &pdsfname: Linkage Status Indicators ";
proc freq data = linked;
format linkstatus linkstatusf. linkstatus_bmi linkstatus_bmif.;
tables linkstatus * linkstatus_bmi / list missing;
run;
title "";

*** de-dup linked dataset to get unique list of cancer survivors by linkage status;
proc sort data=linked out=linked_nodup nodupkey; 
 by MEPSID;
run;

data meps;
 merge pooled_meps00to16 
       linked_nodup (in=a);
 by MEPSID;
 
 *** create indicator for cancer survivors who have same age range and cancer type as clinical trial patients;
 if a then prostate45=1; 
 else prostate45=0;
 
 *** create indicator for which cancer survivors of interest matched with at least one clinical trial patient;
 if linkstatus=1 then matched=1; 
 else if a then matched=2;
 else matched = -9; *recoding as -9 because SAS drops obs with missing values for any variable in a tables statement;
 
 if      prostate45=1 and matched=1 then matched_prostate45 = 1;
 else if prostate45=1 and matched=2 then matched_prostate45 = 2;
 else matched_prostate45 = 0;

 *** create ID for SUDAAN;
 sudcaseID = _n_;
run;

title "Check number of MEPS cancer survivors and linkage status indicator";
title2 "Linkage status based on standard criteria (age, race, sex, and EQ-5D)";
proc freq data=meps;
 format prostate45 prostate45f. matched matchedf. agecat agecatf. racecat racecatf. matched_prostate45 matched_prostate45f.;
 tables prostate45 
        prostate45 * matched 
		prostate45 * matched * matched_prostate45 
        prostate45 * agecat 
        prostate45 * racecat  /list missing;
run;

title "Check number of MEPS years represented for cancer type and age group(e.g., prostate cancer survivors ages 45+)";
title2 "Used to update analysis weights below ";
proc freq data = meps;
tables year 
       age / list missing;
run;
title ""; title2 "";

/* The variable LINKSTATUS represents the outcome of matching PDS patients and MEPS cancer survivors 
   by age, sex, race, and quality of life as scored from the EQ-5D questionnaire. Each MEPS cancer
   survivor either linked with at least one PDS patient or not. For some PDS trials, additional variables   
   such as BMI category were available to match PDS and MEPS cases. The outcome of incorporating the
   additional variable into the matching process is represented by variable LINKSTATUS_BMI. This 
   variable could be used to define linkage outcome for analysis instead of LINKSTATUS as desired. 
   See example code to process LINKSTATUS_BMI below. 

proc means data = linked noprint;
class MEPSID; types MEPSID;
var linkstatus_bmi;
output out = linked_nodup_bmi (drop=_type_ _freq_) min = min_linkstatus_bmi;
run;

data meps;
	merge pooled_meps00to16
	      linked_nodup_bmi (in=a);
	by MEPSID;

   *** create indicator for cancer survivors who have same age range and cancer type as clinical trial patients;
    if a then prostate45=1; 
    else prostate45=0;
 
    if min_linkstatus_bmi = 1 then matched = 1;
	else if a then matched = 2;
	else matched = -9;

    if      prostate45=1 and matched=1 then matched_prostate45 = 1;
    else if prostate45=1 and matched>1 then matched_prostate45 = 2;
    else matched_prostate45 = 0;
run;

title "Check number of MEPS cancer survivors and linkage status indicator";
title2 "Linkage status based on enhanced linkage criteria (age, race, sex, EQ-5D, BMI)";
proc freq data=meps;
 format prostate45 prostate45f. matched matchedf. agecat agecatf. racecat racecatf. matched_prostate45 matched_prostate45f.;
 tables prostate45 
        prostate45 * matched 
		prostate45 * matched * matched_prostate45 
        prostate45 * agecat 
        prostate45 * racecat  /list missing;
run;
*/

data meps_analysis;
 set meps;

 **create the analysis weight for the pooled data 
  (divide by number of MEPS years pooled together in subpop of interest);
 PERWTFP17 = PERWTF/17;
 SAQWTFP17 = SAQWTF/17;
 
 **recode MEPS analysis variables as needed (e.g., Medicare coverage from pooled MEPS dataset);
 if mcarex < 0 then mcarex = .;
 mcarex2 = mcarex;
 if mcarex2 = 2 then mcarex2 = 0;  **recode '2' to '0' for No for PROC SURVEYMEANS;

 if      racecat = 1 then white = 1; 
 else if racecat > 0 then white = 2;  **collapse non-white categories;

run;

title "Check MEPS variable recodes";
proc freq data=meps_analysis;
 format mcarex _mcarexf. racecat racecatf. white whitef.;
 tables mcarex * mcarex2 
	    racecat * white /list missing;
run;
title " ";

************************************************************************************************
* 1. Produce nationally representative estimates of U.S. cancer survivors with same
*    age range and cancer type as the PDS trial of interest 
************************************************************************************************;

*-------------------------------------------------------------
* analysis using SUDAAN
*-------------------------------------------------------------;
/* Data user must have SAS-callable SUDAAN installed to run this example */
proc sort data=meps_analysis; by stra9617 psu9617; run;

TITLE "ESTIMATE OF MEDICARE COVERAGE USING SUDAAN, 2000-2016 MEPS";
TITLE2 "AMONG PROSTATE CANCER SURVIVORS AGES 45+";
proc crosstab data=meps_analysis filetype=sas design=wr ;
 nest     stra9617 psu9617 ; /* variance strata and clusters */
 weight   PERWTFP17;  		 /* a pooled weight to produce annual averages based on combined years of data.*/
 idvar    sudcaseID;
 subpopn  prostate45 = 1; 		 /* e.g., estimates for prostate cancer survivors age 45+ */
 subgroup mcarex / include=nonmissing;
 levels   2 ;
 print    nsum wsum colper secol  /  ;
 tables   mcarex ;
 output   nsum wsum colper secol / filename = sudfile1 replace ;
 run;

*-------------------------------------------------------------
* analysis using SAS
*-------------------------------------------------------------;
TITLE "ESTIMATE OF MEDICARE COVERAGE USING SAS SURVEYMEANS, 2000-2016 MEPS";
TITLE2 "AMONG PROSTATE CANCER SURVIVORS AGES 45+ (PROSTATE45 = 1)";
proc surveymeans data=meps_analysis SUMWGT NOBS MEAN SUM;
    STRATA  stra9617; 	/*variance strata*/
    CLUSTER psu9617; 	/*variance clusters*/
    WEIGHT  PERWTFP17; 	/*pooled weight to produce annual averages based on combined years of data*/
    VAR     mcarex2;  	/*coded in 0/1 for the mean procedure*/
    DOMAIN  prostate45;     /*estimates by subpop (see subpop = 1 for prostate cancer survivors, ages 45+)*/
RUN;

title "ESTIMATE OF MEDICARE COVERAGE USING SAS SURVEYFREQ, 2000-2016 MEPS";
TITLE2 "AMONG PROSTATE CANCER SURVIVORS AGES 45+ (ROW PERCENT FOR PROSTATE45 = 1)";
proc surveyfreq data=meps_analysis;
   tables  prostate45 * mcarex / nowt nototal row; /*estimates by subpop (see subpop = 1 row percent for prostate cancer survivors, ages 45+)*/
   STRATA  stra9617;  	/*variance strata*/
   CLUSTER psu9617; 	/*variance clusters*/
   WEIGHT  PERWTFP17; 	/*pooled weight to produce annual averages based on combined years of data*/
run;

************************************************************************************************
* 2. Produce nationally representative subgroup estimates among U.S. cancer survivors with 
*    same age range and cancer type as the PDS trial of interest (race used as an example)
************************************************************************************************;

*-------------------------------------------------------------
* analysis using SUDAAN
*-------------------------------------------------------------;
proc sort data=meps_analysis; by stra9617 psu9617; run;
 
TITLE "ESTIMATE OF MEDICARE COVERAGE BY RACE USING SUDAAN, 2000-2016 MEPS";
TITLE2 "AMONG PROSTATE CANCER SURVIVORS AGES 45+";
proc crosstab data=meps_analysis filetype=sas design=wr ;
 nest     stra9617 psu9617 ; /*variance strata and clusters*/
 weight   PERWTFP17;  		 /*pooled weight to produce annual averages based on combined years of data*/
 idvar    sudcaseID;
 subpopn  prostate45 = 1; 		 /*e.g., estimates for prostate cancer survivors age 45+ */
 subgroup white mcarex / include=nonmissing;
 levels   2    2 ;			
 print    nsum wsum colper secol  /  ;
 tables   mcarex * white ; 
 output   nsum wsum colper secol / filename = sudfile1 replace ;
 run;

*-------------------------------------------------------------
* analysis using SAS
*-------------------------------------------------------------;
* See reference for domain analysis: 
* https://support.sas.com/documentation//cdl/en/statug/68162/HTML/default/viewer.htm#statug_surveyfreq_details06.htm;

title "ESTIMATES OF MEDICARE COVERAGE BY RACE USING SAS SURVEYFREQ, MEPS 2000-2016";
TITLE2 "AMONG PROSTATE CANCER SURVIVORS AGES 45+ (SEE PROSTATE45 = 1)";
proc surveyfreq data=meps_analysis ;
   tables  prostate45 * white * mcarex  / nowt nototal row; 
   STRATA  stra9617; /*variance strata*/
   CLUSTER psu9617;  /*variance clusters*/
   WEIGHT PERWTFP17; /*pooled weight to produce annual averages based on combined years of data*/
run;


************************************************************************************************
* 3. Produce estimates that compare characteristics of linked and unlinked MEPS cancer
*    survivors with same cancer type and age range as the PDS clinical trial of interest
************************************************************************************************;

*-------------------------------------------------------------
* analysis using SUDAAN
*-------------------------------------------------------------;
*sort the data by the design variables;
proc sort data=meps_analysis; by stra9617 psu9617; run;

TITLE "ESTIMATES OF MEDICARE BY LINKAGE STATUS USING SUDAAN";
TITLE2 "AMONG MEPS PROSTATE CANCER SURVIVORS AGED 45+";
proc crosstab data=meps_analysis filetype=sas design=wr ;
 nest     stra9617 psu9617 / missunit;
 weight   PERWTFP17;
 idvar    sudcaseID;
 subgroup mcarex matched/ include=nonmissing;
 levels   2      2 ;
 subpopn  prostate45=1 / name="All MEPS Prostate Cancer Survivors Aged 45+";
 print    nsum wsum colper secol  /  ;
 tables   mcarex*matched  ;
 output   nsum wsum colper secol / filename = sudfile3 replace ;
 run;
TITLE2 "";
 
*-------------------------------------------------------------
* analysis using SAS
*-------------------------------------------------------------;
TITLE "ESTIMATES OF MEDICARE BY LINKAGE STATUS USING SAS PROC SURVEYMEANS, MEPS 2000-2016";
TITLE2 "AMONG PROSTATE CANCER SURVIVORS AGES 45+ (SEE MATCHED_PROSTATE45 = 1 and MATCHED_PROSTATE45 = 2)";
PROC SURVEYMEANS DATA=meps_analysis SUMWGT NOBS MEAN SUM;
    STRATA  stra9617;  /*variance strata*/
    CLUSTER psu9617;   /*variance cluster*/
    WEIGHT  PERWTFP17; /*pooled weight to produce annual averages based on combined years of data*/
    VAR     mcarex2;   /*medicare analysis variable, coded in 0/1 for the means procedure*/
    DOMAIN  matched_prostate45; 
RUN;

title "ESTIMATES OF MEDICARE BY LINKAGE STATUS USING SAS PROC SURVEYFREQ, MEPS 2000-2016";
TITLE2 "AMONG PROSTATE CANCER SURVIVORS AGES 45+ (SEE PROSTATE45 = 1)";
proc surveyfreq data=meps_analysis;
   tables  prostate45 * matched * mcarex  / nowt nototal row; /*medicare analysis variable, coded in 1/2 for the freq procedure*/
   STRATA  stra9617;  /*variance strata*/
   CLUSTER psu9617;   /*variance clusters*/
   weight  PERWTFP17; /*pooled weight to produce annual averages based on combined years of data.*/
run;