/*---------------------------------------------------------------------------------------------------------------
/* PROGRAM	: Example4_Select_MEPS_Donor
/* AUTHOR	: Jennifer Unangst / Feng Yu
/* DATE		: 08/10/2020
/*
/* PURPOSE	: Many-to-many linkages were permitted between the patients in the PDS trial and MEPS
/*            cancer survivors. Consequently, the PDS-MEPS linked dataset is non-unique by clinical
/*            trial patient. To conduct analyses that assess relationships between variables collected
/*            during the clinical trial and the MEPS variables appended through linkage, users can choose
/*            one of the MEPS cancer survivors linked to each PDS trial patient as a "donor" for MEPS variables. 
/*            This program presents one approach to randomly choosing a donor for each PDS patient who 
/*            had a linkage with MEPS. Results are known to be highly sensitive to the MEPS donor. It is 
/*            advised that data users assess sensitivity before drawing conclusions.  
/*
/* OUTPUT	: analysis1.sas7bdat, analysis2.sas7bdat, analysis3.sas7bdat           
/*---------------------------------------------------------------------------------------------------------------*/
options nofmterr validvarname=v7 compress=yes;

*** specify working directory;
%let basepath = /rtpnfil03/rtpnfil03_vol7/DataSphere2;

*** specify location of pooled MEPS dataset;
libname mepsdat "&basepath/SAS_pgms/05_Data_Deliverable/Prostat_AstraZe_2009_144/SampleCode/data/";

*** specify location of PDS-MEPS linked dataset; 
libname linkdat  "&basepath/Deliver/Prostat_AstraZe_2009_144";

*** specify name of PDS-MEPS linked dataset;
%let pdsfname  = prostate144;

*enter random numbers that will be used to select a MEPS donor for each PDS patient;
%let randnum1 = 34798;
%let randnum2 = 80341;
%let randnum3 = 58925;

*------------------------------------------------------------------------------------------
* READ LINKAGES AND CREATE ANALYSIS FILE UNIQUE BY PDS PATIENT
* ASSIGN SINGLE MEPS DONOR PER PDS CASE
*------------------------------------------------------------------------------------------;
proc format ;
	*per PDS-MEPS linked dataset for definitions;
	value _LINKMETHODF
	 0 = "0 NOT LINKED"
	 1 = "1 CAT AGE, SEX, RACE, DIRECT EQ5D VAL"
	 2 = "2 CAT AGE, SEX, RACE, PRED EQ5D CAT (DOLAN)"
	 3 = "3 CAT AGE, SEX, RACE, PRED EQ5D CAT (SG)"
	 ;
	value _LINKSTATUSF
	 1 = "1 PDS-MEPS LINK"
	 2 = "2 UNLINKED MEPS CASE"
	 ;
run;

proc contents data=linkdat.linked_&pdsfname; run;

title "PDS-MEPS Linked Dataset: &pdsfname";
proc freq data = linkdat.linked_&pdsfname;
format linkstatus _linkstatusf. linkmethod _linkmethodf.;
tables linkstatus * linkmethod / list missing;
tables year * linkmethod / missing;
run;
title "";

*** Create dataset containing all linkages between PDS and MEPS;
data linkages;
	set linkdat.linked_&pdsfname;
	
	if pdsid = ' ' then delete; *drop observations that correspond to unlinked MEPS cancer survivors;
	
	randomnum1 = ranuni(&randnum1);
 	randomnum2 = ranuni(&randnum2);
 	randomnum3 = ranuni(&randnum3);

	pdsid = strip(left(pdsid));

	******************************************************************
	* Update link method ranking as needed for each PDS trial 
	* See documentation accompanying linked dataset for details
	*
	* Example:
 	* linkMethodRank = 1 for cases matched by direct EQ5D score
 	* linkMethodRank = 2 for cases matched by predicted EQ5D classes 
	******************************************************************;

	if      linkmethod in (1   ) then linkMethodRank = 1;
	else if linkmethod in (2 3 ) then linkMethodRank = 2;

	keep mepsid pdsid linkmethodrank linkmethod random:;
run;

proc freq data = linkages;
format linkmethod _linkmethodf.;
title "Check Link Method Ranking (How We Prioritize for Donor Selection)";
tables linkmethod * linkmethodrank / list missing;
run;
title "";

*** Count number of potential MEPS donors by linkage method;
proc sort data = linkages; by pdsid linkmethod; run;

data count_links;
	set linkages;
	by pdsid;

	*** Update based on number of linkage methods based on the trial 
	    (see documentation accompanying PDS-MEPS linked dataset);
	retain nlink1 - nlink3;

	if first.pdsid then do;
		nlink1 = 0;
		nlink2 = 0;
		nlink3 = 0;
	end;

	if linkmethod = 1 then nlink1+1;
	if linkmethod = 2 then nlink2+1;
	if linkmethod = 3 then nlink3+1;

	nlinktot = sum(nlink1, nlink2, nlink3);

	if last.pdsid;
	keep pdsid nlink1 - nlink3 nlinktot;

	label nlinktot = 'Total MEPS Donors Linked to PDS Patient'
	      nlink1   = 'Number of MEPS Donors Linked to PDS Patient Using LinkMethod 1'
	      nlink2   = 'Number of MEPS Donors Linked to PDS Patient Using LinkMethod 2'
	      nlink3   = 'Number of MEPS Donors Linked to PDS Patient Using LinkMethod 3';
run;

title "Number of Candidate MEPS Donors per PDS Patient - Overall and By Linkage Method ";
proc means data = count_links min p25 median p75 max;
var nlinktot nlink1 - nlink3;
run;
title "";

*** Determine strictest set of criteria under which at least one linkage was achieved for each PDS case;
proc means data = linkages noprint;
class pdsid; types pdsid;
var linkmethodrank ;
output out = strictest (drop=_type_ _freq_) min = ;
run;

proc freq data=strictest; 
title "Strictest Linkage Method that Produced a Match with MEPS (Among PDS Patients w/ At Least One MEPS Match)";
tables linkmethodrank / list missing; 
run;
title "";

************************************************************************************
* Construct several donor sets to assess sensitivity of results  
* We provide three donor sets as an example
************************************************************************************;

*** Donor Set 1;
*Randomly choose one donor from those obtained under strictest match criteria;
proc sort data=linkages out=linkages_sort1; by pdsid linkmethodrank randomnum1; run;
data donor1;
 merge linkages_sort1 (in=a)
       strictest      (in=b);
 by pdsid linkmethodrank;
 if a and b;
 if first.pdsid;
 keep pdsid mepsid linkmethodrank;
run;

*** Donor Set 2;
*Randomly choose one donor from those obtained under strictest match criteria;
proc sort data=linkages out=linkages_sort2; by pdsid linkmethodrank randomnum2; run;
data donor2;
 merge linkages_sort2 (in=a)
       strictest      (in=b);
 by pdsid linkmethodrank;
 if a and b;
 if first.pdsid;
 keep pdsid mepsid linkmethodrank;
run;

*** Donor Set 3;
*Choose one donor case completely at random;
proc sort data=linkages out=linkages_sort3; by pdsid randomnum3; run;
data donor3;
 set linkages_sort3;
 by pdsid ;
 if first.pdsid;
 keep pdsid mepsid linkmethodrank;
run;

*check we kept a donor from strictest match criteria;
proc compare base=donor1 compare=strictest;
title "Donor Set #1";
 id pdsid ;
 var linkmethodrank;
run;
proc compare base=donor2 compare=strictest ;
title "Donor Set #2";
 id pdsid ;
 var linkmethodrank;
run;

*check number of times a MEPS cancer survivor is used as a donor in each set;
proc freq data = donor1 noprint; tables mepsid / out = donorfreq1 ; run;
proc freq data = donor2 noprint; tables mepsid / out = donorfreq2 ; run;
proc freq data = donor3 noprint; tables mepsid / out = donorfreq3 ; run;

title "Donor Set #1: Number of Times MEPS Case Used as Donor";
proc univariate data = donorfreq1; var count; run;

title "Donor Set #2: Number of Times MEPS Case Used as Donor";
proc univariate data = donorfreq2; var count; run;

title "Donor Set #3: Number of Times MEPS Case Used as Donor";
proc univariate data = donorfreq3; var count; run;

*------------------------------------------------------------------
* Merge MEPS analysis variables onto PDS patients based on donor
*------------------------------------------------------------------;

*** Users can pull MEPS analysis variables directly from PDS-MEPS linked dataset, or 
    can append additional variables from the MEPS public use files (see Example2 program) ;

proc sort data=linkdat.linked_&pdsfname out=meps_nodup(drop=PDSID) nodupkey; 
 by MEPSID;
run;

proc sort data=donor1; by MEPSID; run;

proc sort data=donor2; by MEPSID; run;

proc sort data=donor3; by MEPSID; run;

data analysis1;
 merge donor1(in=a) meps_nodup(in=b);
 by MEPSID;
 if a;
run;

data analysis2;
 merge donor2(in=a) meps_nodup(in=b);
 by MEPSID;
 if a;
run;

data analysis3;
 merge donor3(in=a) meps_nodup(in=b);
 by MEPSID;
 if a;
run;

*-----------------------------------------------------------------------------------------------
* These example analysis datasets only contain MEPS survey variables. 
* They can be supplemented by appending variables from the datasets
* that accompany the clinical trial, merging by PDSID.
*-----------------------------------------------------------------------------------------------;
